package com.guwan.backend.MergeStrategy;

import java.io.Serializable;
import java.lang.invoke.SerializedLambda;
import java.lang.reflect.Method;
import java.util.function.Function;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Lambda表达式和方法引用解析工具类
 */
public class LambdaUtils {
    
    /**
     * 从Lambda表达式或方法引用中提取属性名
     * @param getter Lambda方法引用
     * @return 属性名
     */
    public static <T, R> String getPropertyName(PropertyFunction<T, R> getter) {
        try {
            // 1. 获取SerializedLambda
            Method writeReplace = getter.getClass().getDeclaredMethod("writeReplace");
            writeReplace.setAccessible(true);
            SerializedLambda serializedLambda = (SerializedLambda) writeReplace.invoke(getter);
            
            // 2. 从SerializedLambda获取实现方法名
            String implMethodName = serializedLambda.getImplMethodName();
            
            // 3. 转换方法名为属性名
            if (implMethodName.startsWith("get") && implMethodName.length() > 3) {
                String propertyName = implMethodName.substring(3);
                return propertyName.substring(0, 1).toLowerCase() + propertyName.substring(1);
            } else if (implMethodName.startsWith("is") && implMethodName.length() > 2) {
                String propertyName = implMethodName.substring(2);
                return propertyName.substring(0, 1).toLowerCase() + propertyName.substring(1);
            }
            
            return implMethodName;
        } catch (Exception e) {
            // 降级到常规正则表达式解析
            return parsePropertyNameByRegex(getter);
        }
    }
    
    /**
     * 使用正则表达式解析属性名（备选方案）
     */
    private static <T, R> String parsePropertyNameByRegex(Function<T, R> getter) {
        // 尝试从Lambda toString()解析
        String methodRef = getter.toString();
        
        // 处理方法引用格式: com.example.Entity::getName
        if (methodRef.contains("::")) {
            Pattern pattern = Pattern.compile("::get(\\w+)|::is(\\w+)");
            Matcher matcher = pattern.matcher(methodRef);
            if (matcher.find()) {
                String fieldName = matcher.group(1) != null ? matcher.group(1) : matcher.group(2);
                return fieldName.substring(0, 1).toLowerCase() + fieldName.substring(1);
            }
            
            // 直接从方法名截取
            String methodName = methodRef.substring(methodRef.lastIndexOf("::") + 2);
            return extractFieldNameFromMethod(methodName);
        } 
        // 处理Lambda表达式格式: (Entity e) -> e.getName() 或 e -> e.getName()
        else if (methodRef.contains("->")) {
            Pattern pattern = Pattern.compile("->\\s*\\w+\\.get(\\w+)\\(\\)|->\\s*\\w+\\.is(\\w+)\\(\\)");
            Matcher matcher = pattern.matcher(methodRef);
            if (matcher.find()) {
                String fieldName = matcher.group(1) != null ? matcher.group(1) : matcher.group(2);
                return fieldName.substring(0, 1).toLowerCase() + fieldName.substring(1);
            }
        }
        
        // 处理其他情况 - 也许是已经是字段名
        return methodRef;
    }
    
    /**
     * 从方法名中提取字段名
     */
    private static String extractFieldNameFromMethod(String methodName) {
        if (methodName.startsWith("get") && methodName.length() > 3) {
            // getName -> name (首字母小写)
            String fieldName = methodName.substring(3);
            return fieldName.substring(0, 1).toLowerCase() + fieldName.substring(1);
        } else if (methodName.startsWith("is") && methodName.length() > 2) {
            // isActive -> active (首字母小写)
            String fieldName = methodName.substring(2);
            return fieldName.substring(0, 1).toLowerCase() + fieldName.substring(1);
        }
        return methodName;
    }
    
    /**
     * 可序列化的函数接口 - 用于支持方法引用解析
     */
    @FunctionalInterface
    public interface PropertyFunction<T, R> extends Function<T, R>, Serializable {
    }
} 