package com.guwan.backend.jpaTest;

import com.guwan.backend.MergeStrategy.JoinBuilder;
import com.guwan.backend.MergeStrategy.JoinBuilder.FieldMapping;
import com.guwan.backend.MergeStrategy.LambdaUtils.PropertyFunction;
import jakarta.persistence.EntityManager;
import jakarta.persistence.PersistenceContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

@Service
public class EmployeeService {

    @Autowired
    private EmployeeRepository employeeRepository;
    
    @PersistenceContext
    private EntityManager entityManager;

    /**
     * 获取员工详细信息，包含部门和项目信息
     * 使用字符串方式指定字段，并直接映射到DTO
     * 
     * @param departmentName 部门名称过滤条件
     * @param projectDeleted 项目是否已删除的过滤条件
     * @return 员工DTO列表
     */
    public List<EmployeeDTO> getEmployeeDetails(String departmentName, boolean projectDeleted) {
        // 创建查询构建器，从Employee实体开始
        JoinBuilder<Employee> builder = JoinBuilder.from(Employee.class);

        // 添加关联，使用字符串指定实体间的关系字段
        builder.join(Department.class, "departmentId", "id");
        builder.join(Project.class, "projectId", "id");

        // 添加查询条件，使用字符串指定字段
        if (departmentName != null && !departmentName.isEmpty()) {
            builder.equal(Department.class, "name", departmentName);
        }
        builder.equal(Project.class, "deleted", projectDeleted);

        // 使用selectAs设置字段别名，避免使用FieldMapping
        builder.selectAs("employeeId", Employee.class, "id");
        builder.selectAs("employeeName", Employee.class, "name");
        builder.selectAs("departmentName", Department.class, "name");
        builder.selectAs("projectName", Project.class, "name");

        // 直接执行查询并映射到DTO
        return builder.executeAndMap(entityManager, EmployeeDTO.class);
    }
    
    /**
     * 使用自定义条件查询员工信息
     * 使用新的方法引用方式
     * 
     * @param employeeName 员工名称过滤条件，可选
     * @param departmentName 部门名称过滤条件，可选
     * @return 员工DTO列表
     */
    public List<EmployeeDTO> findEmployees(String employeeName, String departmentName) {
        JoinBuilder<Employee> builder = JoinBuilder.from(Employee.class);
        
        // 尝试使用增强的方法引用
        try {
            // 添加关联，使用增强的方法引用
            builder.join(Department.class, 
                         (PropertyFunction<Employee, Long>) Employee::getDepartmentId, 
                         (PropertyFunction<Department, Long>) Department::getId);
            builder.join(Project.class, 
                         (PropertyFunction<Employee, Long>) Employee::getProjectId, 
                         (PropertyFunction<Project, Long>) Project::getId);
            
            // 添加查询条件，使用增强的方法引用
            if (employeeName != null && !employeeName.isEmpty()) {
                builder.equal(Employee.class, 
                              (PropertyFunction<Employee, String>) Employee::getName, 
                              employeeName);
            }
            
            if (departmentName != null && !departmentName.isEmpty()) {
                builder.equal(Department.class, 
                              (PropertyFunction<Department, String>) Department::getName, 
                              departmentName);
            }
            
            // 使用selectAs配合增强的方法引用
            builder.selectAs("employeeId", Employee.class, 
                           (PropertyFunction<Employee, Long>) Employee::getId);
            builder.selectAs("employeeName", Employee.class, 
                           (PropertyFunction<Employee, String>) Employee::getName);
            builder.selectAs("departmentName", Department.class, 
                           (PropertyFunction<Department, String>) Department::getName);
            builder.selectAs("projectName", Project.class, 
                           (PropertyFunction<Project, String>) Project::getName);
        }
        catch (Exception e) {
     /*       // 如果增强方法引用失败，回退到字符串方式
            builder = JoinBuilder.from(Employee.class);

            // 使用字符串作为回退方案
            builder.join(Department.class, "departmentId", "id");
            builder.join(Project.class, "projectId", "id");

            if (employeeName != null && !employeeName.isEmpty()) {
                builder.equal(Employee.class, "name", employeeName);
            }

            if (departmentName != null && !departmentName.isEmpty()) {
                builder.equal(Department.class, "name", departmentName);
            }

            builder.selectAs("employeeId", Employee.class, "id");
            builder.selectAs("employeeName", Employee.class, "name");
            builder.selectAs("departmentName", Department.class, "name");
            builder.selectAs("projectName", Project.class, "name");*/
        }
        
        // 直接执行查询并映射到DTO
        return builder.executeAndMap(entityManager, EmployeeDTO.class);
    }
} 